@@ -238,7 +238,14 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
     let circuit = timed!(compiler_simp(&circuit, context), 10, print_timings);
     // originally tried push_down_index in compiler_simp, but that produced worse circuits
     // for unknown reasons, maybe i'll investigate
-    let circuit = timed!(deep_push_down_index(circuit, None), 10, print_timings);
+    let circuit = timed!(
+        apply_fn_until_same(&circuit, |x| deep_push_down_index(
+            compiler_simp(&x, context),
+            None
+        )),
+        10,
+        print_timings
+    );
     let circuit = timed!(compiler_simp(&circuit, context), 10, print_timings);
     let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
     if context.settings.verbose >= 3 {
