@@ -63,8 +63,8 @@ pub fn assert_is_close(a: CircuitRc, b: CircuitRc) {
     let close_result = is_close(a_ten.clone(), b_ten.clone());
     if close_result.is_err() {
         println!("{:?}", close_result);
-        println!("LHS: {}", rust_expression_notation_circuit(a));
-        println!("RHS: {}", rust_expression_notation_circuit(b));
+        println!("LHS: {}", rust_expression_notation_circuit(a, true));
+        println!("RHS: {}", rust_expression_notation_circuit(b, true));
         python_print(a_ten.tensor());
         python_print(b_ten.tensor());
         panic!("circuits did not evaluate to the same tensors");
@@ -149,8 +149,8 @@ pub fn create_circuit_test(
         a.info().hash_usize(),
         b.info().hash_usize()
     );
-    let a_str = rust_expression_notation_circuit(a);
-    let b_str = rust_expression_notation_circuit(b);
+    let a_str = rust_expression_notation_circuit(a, true);
+    let b_str = rust_expression_notation_circuit(b, true);
     let test = format!(
         "#[test]\
         \nfn {}() {{\
