@@ -6,7 +6,7 @@ use crate::circuit::{
     deep_map_op_context, generalfunction_merge_inverses,
     generalfunction_special_case_simplification,
 };
-use crate::util::{apply_fn_until_same, mapping_until_end, AsOp};
+use crate::util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, AsOp};
 use crate::{pyo3_prelude::*, timed_value};
 use num_bigint::BigUint;
 
@@ -235,8 +235,11 @@ pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> Circuit
                 .chain(std::iter::once(circ.clone().rc()))
                 .any(|z| z.info().numel() >= BigUint::from(min_size.unwrap()))
         {
-            circ.and_then_or_clone(&|index: &Index| {
-                push_down_index(&index.and_then_or_clone(index_fuse))
+            circ.map_or_clone(&|index: &Index| {
+                let fused = apply_fn_until_none(index, index_fuse);
+                index_elim_identity(&fused).unwrap_or_else(|| {
+                    push_down_index(&fused).unwrap_or_else(|| fused.clone().rc())
+                })
             })
         } else {
             circ.clone().rc()
