@@ -20,6 +20,7 @@ pub use generalfunction_rewrite::{
 };
 pub use module_nodes::{ModuleNode, ModuleNodeArgSpec, ModuleNodeSpec};
 use num_bigint::BigUint;
+pub use parsing::parse_compiler_repr_bijection;
 use py_circuit_items::circuit_rust_to_py;
 use pyo3::{exceptions, pyclass::CompareOp};
 use std::hash::Hash;
@@ -53,6 +54,7 @@ pub mod generalfunction_rewrite;
 pub mod module_nodes;
 pub mod named_axes;
 pub mod nrc;
+mod parsing;
 pub mod print;
 pub mod py_circuit_items;
 mod repr;
@@ -298,7 +300,7 @@ pub trait CircuitNode: CircuitNodePrivate {
     where
         Self: Into<Circuit> + Clone,
     {
-        repr_circuit_deep_compiler(&self.clone().c())
+        repr_circuit_deep_compiler(&self.clone().c(), false, false)
     }
 
     fn compiler_print(&self)
@@ -794,6 +796,9 @@ pub enum CircuitConstructionError {
     #[error("Invalid rearrange spec {spec:?}")]
     InvalidRearrangeSpec { spec: RearrangeSpec },
 
+    #[error("Rearrange String Invalid {string}")]
+    RearrangeStringInvalid { string: String },
+
     #[error("Wrong input shapes for GeneralFunction {input_shapes:?} {gf_spec:?}")]
     GeneralFunctionWrongInputShape {
         gf_spec: GeneralFunctionSpec,
@@ -888,6 +893,36 @@ pub enum CircuitConstructionError {
     #[error("ModuleNode got unknown keyword argument, {argument}")]
     ModuleNodeUnknownArgument { argument: String },
 
+    #[error("Parsing no regex match '{line}'")]
+    ParsingNoRegexMatch { line: String },
+
+    #[error("Parsing number failed {string}")]
+    ParsingNumberFail { string: String },
+
+    #[error("Parsing failed {string}")]
+    ParsingFail { string: String },
+
+    #[error("Parsing invalid indentation, number of spaces {spaces}")]
+    ParsingInvalidIndentation { spaces: usize },
+
+    #[error("Parsing invalid circuit variant {v}")]
+    ParsingInvalidVariant { v: String },
+
+    #[error("Parsing invalid serial number {serial_number}")]
+    ParsingInvalidSerialNumber { serial_number: usize },
+
+    #[error("Tensor hash not found {hash}")]
+    TensorHashNotFound { hash: String },
+
+    #[error("Parsing Extra Unneeded String '{string}'")]
+    ParsingExtraUnneededString { string: String },
+
+    #[error("Parsing shape needed but not provided on {variant}")]
+    ParsingShapeNeeded { variant: String },
+
+    #[error("Parsing found multiple circuits in sequence, instead of one circuit (aka last node wasn't child of anything)")]
+    ParsingGotMultipleCircuits {},
+
     #[error("python error {py_err:?}")]
     PythonError { py_err: Rc<PyErr> },
 }
@@ -1385,12 +1420,14 @@ impl PyCircuitBase {
         Ok(out)
     }
 
-    fn compiler_repr(&self) -> String {
-        repr_circuit_deep_compiler(self)
+    #[args(bijection = "false", shape_only_necessary = "false")]
+    fn compiler_repr(&self, bijection: bool, shape_only_necessary: bool) -> String {
+        repr_circuit_deep_compiler(self, bijection, shape_only_necessary)
     }
 
-    fn compiler_print(&self) {
-        println!("{}", self.compiler_repr())
+    #[args(bijection = "false", shape_only_necessary = "false")]
+    fn compiler_print(&self, bijection: bool, shape_only_necessary: bool) {
+        println!("{}", self.compiler_repr(bijection, shape_only_necessary))
     }
 
     fn numel(&self) -> BigUint {
