@@ -2,6 +2,7 @@ use crate::hashmaps::FxHashMap as HashMap;
 use crate::py_types::PY_UTILS;
 use crate::rrfs::{save_tensor_rrfs, tensor_from_hash_prefix};
 use crate::smallvec::Sv;
+use crate::util::cumsum;
 use crate::{
     circuit::{CircuitConstructionError, HashBytes},
     filter_by_variant,
@@ -314,6 +315,50 @@ impl TensorAxisIndex {
         false
     }
 
+    pub fn split_sections_for_concat(
+        &self,
+        sections: &Vec<usize>,
+    ) -> Option<Vec<Option<TensorAxisIndex>>> {
+        let starts = cumsum(sections);
+        match self {
+            TensorAxisIndex::Single(single) => {
+                if single < &0 {
+                    return None;
+                }
+                let outer_idx = starts
+                    .iter()
+                    .take_while(|start| **start <= *single as usize)
+                    .count()
+                    - 1;
+                let inner_idx = *single as usize - starts[outer_idx];
+                let mut result = vec![None; sections.len()];
+                result[outer_idx] = Some(TensorAxisIndex::Single(inner_idx as i64));
+                Some(result)
+            }
+            TensorAxisIndex::Slice(slice) => {
+                let uslice: Option<USlice> = slice.clone().into();
+                let uslice = uslice?;
+                let mut result = vec![None; sections.len()];
+                let mut did_any = false;
+                for (i, (start, sec_len)) in zip(starts.iter(), sections.iter()).enumerate() {
+                    let stop = start + sec_len;
+                    if *start < uslice.stop && stop > uslice.start {
+                        did_any = true;
+                        result[i] = Some(TensorAxisIndex::new_plain_slice(
+                            uslice.start.saturating_sub(*start),
+                            min(*sec_len, uslice.stop - start),
+                        ))
+                    }
+                }
+                if !did_any {
+                    return None;
+                }
+                Some(result)
+            }
+            TensorAxisIndex::Tensor(_tensor) => None,
+        }
+    }
+
     pub fn new_tensor_randint_seeded(
         length: usize,
         base_length: usize,
