@@ -241,6 +241,24 @@ where
     result
 }
 
+pub fn apply_fn_until_none<T: Clone, F>(x: &T, f: F) -> T
+where
+    F: FnMut(&T) -> Option<T>,
+{
+    let mut f = f;
+    let mut result = x.clone();
+    for i in 0..1000 {
+        match f(&result) {
+            None => return result,
+            Some(new) => result = new,
+        }
+        if i == 1000 - 1 {
+            panic!("apply until same didnt finish");
+        }
+    }
+    result
+}
+
 // might be fun to make memcpy based optimized outer product where we just copy blocks into place
 pub fn outer_product<T: Clone>(cols: &Vec<Vec<T>>) -> Vec<Vec<T>> {
     assert!(!cols.iter().any(|x| x.is_empty()));
